@@ -264,6 +264,10 @@ sub merge_specs {
           ($old_spec->{$key}, $spec->{$key})
         ];
       }
+      elsif ($key eq 'builder' || $key eq 'default') {
+        $spec->{$key} = $old_spec->{$key}
+          if !(exists $spec->{builder} || exists $spec->{default});
+      }
       elsif (!exists $spec->{$key}) {
         $spec->{$key} = $old_spec->{$key};
       }
@@ -681,9 +685,7 @@ sub _validate_codulatable {
     $error = "could not be converted to a coderef: $@";
   }
   elsif (ref $value eq 'CODE') {
-    defined &$value
-      and return 1;
-    $error = 'is a stub';
+    return 1;
   }
   else {
     $error = 'is not a coderef or code-convertible object';
@@ -691,7 +693,7 @@ sub _validate_codulatable {
 
   croak "Invalid $setting '"
     . ($INC{'overload.pm'} ? overload::StrVal($value) : $value)
-    . "' for $into" . $error
+    . "' for $into " . $error
     . ($appended ? " $appended" : '');
 }
 
