@@ -22,26 +22,70 @@ BEGIN {
 
 use Module::Runtime qw(use_package_optimistically module_notional_filename);
 
-use Devel::GlobalDestruction ();
 use Exporter qw(import);
 use Config;
+use Scalar::Util qw(weaken);
 use Carp qw(croak);
 
+# this should be empty, but some CPAN modules expect these
 our @EXPORT = qw(
-    _getglob _install_modifier _load_module _maybe_load_module
-    _getstash _install_coderef _name_coderef
-    _unimport_coderefs _set_loaded
+  _install_coderef
+  _load_module
 );
 
-sub _install_modifier {
-  my ($into, $type, $name, $code) = @_;
+our @EXPORT_OK = qw(
+  _check_tracked
+  _getglob
+  _getstash
+  _install_coderef
+  _install_modifier
+  _install_tracked
+  _load_module
+  _maybe_load_module
+  _name_coderef
+  _set_loaded
+  _unimport_coderefs
+);
 
-  if ($INC{'Sub/Defer.pm'} and my $to_modify = $into->can($name)) { # CMM will throw for us if not
-    Sub::Defer::undefer_sub($to_modify);
+my %EXPORTS;
+
+sub _install_modifier {
+  my $target = $_[0];
+  my $type = $_[1];
+  my $code = $_[-1];
+  my @names = @_[2 .. $#_ - 1];
+
+  @names = @{ $names[0] }
+    if ref($names[0]) eq 'ARRAY';
+
+  my @tracked = _check_tracked($target, \@names);
+
+  if ($INC{'Sub/Defer.pm'}) {
+    for my $name (@names) {
+      # CMM will throw for us if it doesn't exist
+      if (my $to_modify = $target->can($name)) {
+        Sub::Defer::undefer_sub($to_modify);
+      }
+    }
   }
 
   require Class::Method::Modifiers;
   Class::Method::Modifiers::install_modifier(@_);
+
+  if (@tracked) {
+    my $exports = $EXPORTS{$target};
+    weaken($exports->{$_} = $target->can($_))
+      for @tracked;
+  }
+
+  return;
+}
+
+sub _install_tracked {
+  my ($target, $name, $code) = @_;
+  my $from = caller;
+  weaken($EXPORTS{$target}{$name} = $code);
+  _install_coderef("${target}::${name}", "${from}::${name}", $code);
 }
 
 sub _load_module {
@@ -99,23 +143,41 @@ sub _name_coderef {
   _CAN_SUBNAME ? _subname(@_) : $_[1];
 }
 
-sub _unimport_coderefs {
-  my ($target, $info) = @_;
-  return unless $info and my $exports = $info->{exports};
-  my %rev = reverse %$exports;
+sub _check_tracked {
+  my ($target, $names) = @_;
   my $stash = _getstash($target);
-  foreach my $name (keys %$exports) {
-    if ($stash->{$name} and defined(&{$stash->{$name}})) {
-      if ($rev{$target->can($name)}) {
-        my $old = delete $stash->{$name};
-        my $full_name = join('::',$target,$name);
-        # Copy everything except the code slot back into place (e.g. $has)
-        foreach my $type (qw(SCALAR HASH ARRAY IO)) {
-          next unless defined(*{$old}{$type});
-          no strict 'refs';
-          *$full_name = *{$old}{$type};
-        }
-      }
+  my $exports = $EXPORTS{$target}
+    or return;
+
+  $names = [keys %$exports]
+    if !$names;
+  my %rev =
+    map +($exports->{$_} => $_),
+    grep defined $exports->{$_},
+    keys %$exports;
+
+  return
+    grep {
+      my $g = $stash->{$_};
+      $g && defined &$g && exists $rev{\&$g};
+    }
+    @$names;
+}
+
+sub _unimport_coderefs {
+  my ($target) = @_;
+
+  my $stash = _getstash($target);
+  my @exports = _check_tracked($target);
+
+  foreach my $name (@exports) {
+    my $old = delete $stash->{$name};
+    my $full_name = join('::',$target,$name);
+    # Copy everything except the code slot back into place (e.g. $has)
+    foreach my $type (qw(SCALAR HASH ARRAY IO)) {
+      next unless defined(*{$old}{$type});
+      no strict 'refs';
+      *$full_name = *{$old}{$type};
     }
   }
 }
